///
/// @file:			applet.hh
/// author:			E^rthW( )rm 2003
/// last mod:			Friday, September 05 2003
/// about:			declares the c_applet class
/// license:			GPL

#ifndef _APPLET_HH_
#define _APPLET_HH_

// includes //
//
#include <memory>
#include <list>
#include <sigc++/connection.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/window.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/image.h>
#include <glibmm/ustring.h>
#include <gconfmm/client.h>
#include <gconfmm/entry.h>
#include <libgnomeuimm/about.h>
#include <panel-applet.h>
#include "wallpapers.hh"
#include "search_dialog.hh"
#include "preferences_dialog.hh"

// class c_applet //
//
/// applet object which controls the program
class c_applet : public Gtk::EventBox
{
	// public class interface //
	//
	public:
		c_applet			(PanelApplet			* p_panel_applet);

		~c_applet			();

		Gtk::Container &
		get_container			();
  
		/// size of panel applet in pixels
		int
		get_size			() const;

		/// whether we're in horizontal mode
		bool
		horizontal			() const;

	// private class interface //
	//
	private:
		/// handles next wallpaper selection from the menu
		void
		on_forward_activated		();

		/// handles wallpaper search selection from the menu
		void
		on_search_activated		();

		/// handles delete wallpaper selection from the menu
		void
		on_delete_activated		();

		/// handles preferences selection from the menu
		void
		on_preferences_activated	();

		/// handles help selection from the menu
		void
		on_help_activated		();

		/// handles about selection from the menu
		void
		on_about_activated		();

		/// handles dnd drop events
		void
		on_drop_drag_data_received	(const Glib::RefPtr<Gdk::DragContext>& context,
						 int				n_x,
						 int				n_y,
						 const Gtk::SelectionData	& selection_data,
						 guint				,
						 guint				n_time);

		void
		render_thumbnail		(guint				n_cnxn,
						 Gnome::Conf::Entry		gconf_entry);

		void
		create_tooltip			(guint				n_cnxn,
						 Gnome::Conf::Entry		gconf_entry);

	// private class data //
	//
	private:
		std::auto_ptr<Gnome::UI::About> m_p_about_box;
		
		Gtk::Image::Image		m_gtk_image;
		Gtk::Tooltips			m_gtk_tooltips;

		Glib::RefPtr<Gdk::Pixbuf>	m_ref_gdk_icon;
		Glib::RefPtr<Gnome::Conf::Client> m_ref_gconf_client;

		PanelApplet			* m_p_panel_applet;
		c_preferences_dialog 		* m_p_preferences_dialog;
		c_search_dialog 		* m_p_search_dialog;

		guint				m_n_cnxn_render_thumb;
		guint				m_n_cnxn_update_tooltip;

		c_wallpapers			m_wallpapers;

		friend void 
		display_forward			(BonoboUIComponent		*, 
						 void				* p_applet,
						 const gchar			*);

		friend void 
		display_search			(BonoboUIComponent		*, 
						 void				* p_applet,
						 const gchar			*);

		friend void 
		display_delete			(BonoboUIComponent		*, 
						 void				* p_applet,
						 const gchar			*);

		friend void 
		display_preferences		(BonoboUIComponent		*, 
						 void				* p_applet,
						 const gchar			*);

		friend void
		display_help			(BonoboUIComponent		*, 
						 void 				* p_applet,
						 const gchar			*);

		friend void
		display_about			(BonoboUIComponent		*,
						 void				* p_applet,
						 const gchar			*);
};// end class c_about

#endif	// end _APPLET_HH_
