//
// file:	search_dialog.cc
// author:	earthworm
// date:	Wednesday, 27 April 2005
// about:	defines c_search_dialog class
//

// includes //
//
#include "search_dialog.hh"
#include <stdexcept>
#include <iostream>
#include <glib/gi18n.h>
#include "boost/filesystem/exception.hpp"

// definitions //
//

c_search_dialog::c_search_dialog		(BaseObjectType				* p_window,
						 const Glib::RefPtr<Gnome::Glade::Xml>	& ref_glade) :
	Gtk::Dialog(p_window),
	m_ref_glade(ref_glade)
{
	// set up the gconf client
	m_ref_gconf_client = Gnome::Conf::Client::get_default_client();

	// grab the main druid
	m_ref_glade->get_widget("druid_search", m_p_druid);

	if(m_p_druid != NULL)
	{
		// hook up the cancel and finish buttons
		m_p_druid->signal_cancel().connect(sigc::mem_fun(*this, &c_search_dialog::on_button_cancel_click));
	}// end if

	// grab the finish druid page
	m_ref_glade->get_widget("druidpagefinish", m_p_druid_page_finish);
	if(m_p_druid_page_finish != NULL)
	{
		// hook up the cancel and finish buttons	
		m_p_druid_page_finish->signal_finish().connect(sigc::mem_fun(*this, &c_search_dialog::on_button_apply_click));
	}// end if

	// grab the finish druid page
	m_ref_glade->get_widget("druidpagestandard_search_string", m_p_druid_page_std_search_string);
	if(m_p_druid_page_std_search_string != NULL)
	{
		// hook up the cancel and finish buttons	
		m_p_druid_page_std_search_string->signal_next().connect(sigc::mem_fun(*this, &c_search_dialog::on_button_next_click));
	}// end if

	// grab the finish druid page
	m_ref_glade->get_widget("druidpagestart", m_p_druid_page_std_start);

	// grab the tree view
	m_ref_glade->get_widget("treeview_results", m_p_treeview);

	if(m_p_treeview == NULL)
	{
		throw std::runtime_error("c_search_dialog::c_search_dialog() : failed to get widget from glade");
	}// end if

	// set up the list store and model etc
	m_ref_liststore = Gtk::ListStore::create(m_model);

	m_p_treeview->set_model(m_ref_liststore);
	m_p_treeview->append_column(_("Directories"), m_model.directory_name);

	m_ref_glade->get_widget("radiobutton_select", m_p_radio_button_select);
	m_ref_glade->get_widget("radiobutton_delete", m_p_radio_button_delete);
	m_ref_glade->get_widget("entry_search_text", m_p_entry_search_string);

	// give the window an icon
	set_icon_from_file(GNOMEICONDIR"/wp_tray-applet.png");

	show_all_children();
}// end c_search_dialog constructor

c_search_dialog::~c_search_dialog		()
{
}// end c_search_dialog destructor

bool
c_search_dialog::on_button_next_click		(Gtk::Widget				& ref_druid)
{
	std::list<Glib::ustring> ls_wp_dir, ls_results;

	ls_wp_dir = m_ref_gconf_client->get_string_list("/apps/wp_tray/dir_list");

	boost::regex regex;
	Glib::ustring rstring;

	try {
	    rstring += ".*";
	    rstring += m_p_entry_search_string->get_text();
	    rstring += ".*";
	    regex = boost::regex(rstring.c_str(), boost::regex::perl | boost::regex::icase);
	} catch (...) {
	    Gtk::MessageDialog d(_("Incorrect regular expression.\n"
				   "Please use Perl-style format."), false,
				 Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
	    d.run();
	    return true;
	}

	// search each directory
	for(std::list<Glib::ustring>::iterator it = ls_wp_dir.begin(); it != ls_wp_dir.end(); ++ it)
	{
		// do the boost regex search
		// TODO: C string conversion is 'orrible :/
		c_search_dialog::wallpaper_search(fs::path(*it), regex, ls_results);
	}// end for

	if (ls_results.size() == 0) {
	    Gtk::MessageDialog d(_("\nNo wallpapers found."), false, Gtk::MESSAGE_INFO,
				 Gtk::BUTTONS_OK, true);
	    d.run();
	    return true;
	}

	// clear list store
	m_ref_liststore->clear();

	// append results to the list store
	for(std::list<Glib::ustring>::iterator it = ls_results.begin(); it != ls_results.end(); ++ it)
	{
		Gtk::TreeModel::iterator it_new_row = m_ref_liststore->append();

		(*it_new_row)[m_model.directory_name] = (*it);
	}// end for

	return false;
}// end c_search_dialog::on_button_next_click

void
c_search_dialog::on_button_apply_click		(Gtk::Widget				& ref_widget)
{
	Glib::RefPtr<Gtk::TreeSelection> refptr_tree_selection = m_p_treeview->get_selection();

	Gtk::TreeModel::iterator it = refptr_tree_selection->get_selected();
	
	// if anything is selected
	if(it)
	{
		Glib::ustring sz_value;

		it->get_value(0, sz_value);

		if(m_p_radio_button_delete->get_active() == true)
		{
			if(sz_value.length() == 0)
				return;

			// attempt to remove the file
			try
			{
				fs::path path_wp(sz_value.c_str(), fs::native);

				if(fs::exists(path_wp) == false)
					return;

				fs::remove(path_wp);
			}// end try
			catch(fs::filesystem_error e)
			{
				return;
			}// end catch

			if(m_ref_gconf_client->get_string("/desktop/gnome/background/picture_filename") == sz_value)
			{
				// TODO: set new wp ???
			}// end if
		}// end if
		else
		if(m_p_radio_button_select->get_active() == true && sz_value.length() > 0)
		{
			// set new in gconf
			m_ref_gconf_client->set("/desktop/gnome/background/picture_filename", sz_value);
		}// end if
	}// end if

	c_search_dialog::hide();
}// end c_search_dialog::on_button_apply_click

void
c_search_dialog::on_button_cancel_click		()
{
	c_search_dialog::hide();
}// end c_search_dialog::on_button_cancel_click

void
c_search_dialog::on_show			()
{
	// clear list store
	m_ref_liststore->clear();

	// clear search box
	m_p_entry_search_string->set_text("");

	// TODO: this doesn't work
	m_p_druid->set_page(*m_p_druid_page_std_start);

	Gtk::Dialog::on_show();
}// end c_search_dialog::on_realise

void
c_search_dialog::wallpaper_search		(fs::path				path_current,
						 const boost::regex			& ref_regex,
						 std::list<Glib::ustring>		& ref_ls_results)
{
	fs::directory_iterator end_iter;

	for(fs::directory_iterator dir_itr(path_current); dir_itr != end_iter; ++ dir_itr)
	{
		try
		{
			if(fs::is_directory(*dir_itr))
			{
				// recurse
				c_search_dialog::wallpaper_search(*dir_itr, ref_regex, ref_ls_results);
			}// end if
			else
			{
				if(boost::regex_match(dir_itr->leaf(), ref_regex))
				{
					ref_ls_results.push_back((*dir_itr).native_directory_string());
				}// end if
			}// end else
		}// end try
		catch(const std::exception & ex)
		{
			std::cerr << "c_wallpapers::count_entries_in_directory() : " << dir_itr->leaf() << " " << ex.what() << std::endl;
		}// end catch
	}// end for
}// end c_search_dialog::wallpaper_search
